// SPDX-License-Identifier: MIT
/*
 * Copyright (c) 2023 Atmark Techno,Inc.
 */
#include <assert.h>
#include "../security_imx6ull.h"

int test_boot_cfg_lock_settings_imx6ull(void)
{
	char *status_char;
	uint32_t status_mem;

	status_mem = IMX6ULL_BOOT_CFG_LOCK_OPWP;
	status_char = boot_cfg_lock_settings_imx6ull(status_mem);
	assert((strcmp(status_char, "write protected") == 0));

	status_mem = 0x8;
	status_char = boot_cfg_lock_settings_imx6ull(status_mem);
	assert((strcmp(status_char, "open") == 0));

	return 0;
}

int test_jtag_settings_imx6ull(void)
{
	char *status_char;
	uint32_t status_mem;

	status_mem = IMX6ULL_SJC_DISABLE_TRUE
	    | IMX6ULL_JTAG_SMODE_SECURE
	    | IMX6ULL_KTE_KILL;
	status_char = jtag_settings_imx6ull(status_mem);
	assert((strcmp(status_char, "disabled") == 0));

	status_mem = IMX6ULL_JTAG_SMODE_JTAG_ENABLE | IMX6ULL_KTE_KILL;
	status_char = jtag_settings_imx6ull(status_mem);
	assert((strcmp(status_char, "open") == 0));

	// JTAG_SMODE is "reserved" mode.
	status_mem = 0x800000 | IMX6ULL_KTE_KILL;
	status_char = jtag_settings_imx6ull(status_mem);
	assert((strcmp(status_char, "unknown") == 0));

	status_mem = IMX6ULL_JTAG_SMODE_SECURE;
	status_char = jtag_settings_imx6ull(status_mem);
	assert((strcmp(status_char, "secure mode. WARNING: bus tracing is allowed") == 0));

	status_mem = IMX6ULL_JTAG_SMODE_SECURE | IMX6ULL_KTE_KILL;
	status_char = jtag_settings_imx6ull(status_mem);
	assert((strcmp(status_char, "secure mode") == 0));

	status_mem = IMX6ULL_JTAG_SMODE_NO_DEBUG;
	status_char = jtag_settings_imx6ull(status_mem);
	assert((strcmp(status_char, "bus tracing only allowed") == 0));

	status_mem = IMX6ULL_JTAG_SMODE_NO_DEBUG | IMX6ULL_KTE_KILL;
	status_char = jtag_settings_imx6ull(status_mem);
	assert((strcmp(status_char, "disabled") == 0));

	return 0;
}

int test_sd_boot_settings_imx6ull(void)
{
	char *status_char;
	uint32_t bootdev_usdhc_bus_mem;
	uint32_t btFuseSel_mem;

	bootdev_usdhc_bus_mem = IMX6ULL_BOOT_DEVICE_EMMC
		| IMX6ULL_USDHC_EMMC
		| IMX6ULL_BUS_WIDTH_8BIT;
	btFuseSel_mem = 0x0;

	status_char = sd_boot_settings_imx6ull(
		bootdev_usdhc_bus_mem,
		btFuseSel_mem);

	assert((strcmp(status_char, "allowed") == 0));

	bootdev_usdhc_bus_mem = IMX6ULL_BOOT_DEVICE_EMMC
		| 0x800
		| IMX6ULL_BUS_WIDTH_8BIT;
	btFuseSel_mem = IMX6ULL_BT_FUSE_SEL_PROGRAMMED;

	status_char = sd_boot_settings_imx6ull(
		bootdev_usdhc_bus_mem,
		btFuseSel_mem);

	assert((strcmp(status_char, "unknown") == 0));

	bootdev_usdhc_bus_mem = 0xC0
		| IMX6ULL_USDHC_EMMC
		| IMX6ULL_BUS_WIDTH_8BIT;
	btFuseSel_mem = IMX6ULL_BT_FUSE_SEL_PROGRAMMED;

	status_char = sd_boot_settings_imx6ull(
		bootdev_usdhc_bus_mem,
		btFuseSel_mem);

	assert((strcmp(status_char, "unknown") == 0));

	bootdev_usdhc_bus_mem = IMX6ULL_BOOT_DEVICE_EMMC
		| IMX6ULL_USDHC_EMMC
		| 0xC000;
	btFuseSel_mem = IMX6ULL_BT_FUSE_SEL_PROGRAMMED;

	status_char = sd_boot_settings_imx6ull(
		bootdev_usdhc_bus_mem,
		btFuseSel_mem);

	assert((strcmp(status_char, "disabled. WARNING: eMMC bus width is not 8-bit") == 0));

	bootdev_usdhc_bus_mem = IMX6ULL_BOOT_DEVICE_EMMC
		| IMX6ULL_USDHC_EMMC
		| IMX6ULL_BUS_WIDTH_8BIT;
	btFuseSel_mem = IMX6ULL_BT_FUSE_SEL_PROGRAMMED;

	status_char = sd_boot_settings_imx6ull(
		bootdev_usdhc_bus_mem,
		btFuseSel_mem);

	assert((strcmp(status_char, "disabled") == 0));

	return 0;
}

int main(void)
{
	printf("-------- test security_imx6ull ---------\n");
	test_boot_cfg_lock_settings_imx6ull();
	printf("lock: ok\n");
	test_jtag_settings_imx6ull();
	printf("jtag: ok\n");
	test_sd_boot_settings_imx6ull();
	printf("sd_boot: ok\n");

	return 0;
}
